/* Copyright 2011, Martin Koller, kollix@aon.at

  This file is part of ShowImage.

  ShowImage is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  ShowImage is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with ShowImage.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <ImageWidget.hxx>
#include <Overlay.hxx>

#include <kurl.h>
#include <kmenu.h>
#include <klocale.h>
#include <kapplication.h>
#include <kaction.h>
#include <kstandardaction.h>
#include <ktogglefullscreenaction.h>
#include <kaboutapplicationdialog.h>
#include <kcmdlineargs.h>
#include <kpropertiesdialog.h>
#include <kfiledialog.h>
#include <kimageio.h>
#include <kactioncollection.h>
#include <kshortcutsdialog.h>
#include <kmessagebox.h>
#include <kstringhandler.h>

#include <QPainter>
#include <QResizeEvent>
#include <QDesktopWidget>
#include <QTimer>
#include <QClipboard>
#include <QSignalMapper>

#define transformationMode Qt::SmoothTransformation

//--------------------------------------------------------------------------------

ImageWidget::ImageWidget(QWidget *parent)
  : QWidget(parent), menu(0), wheelToBrowse(false),
    mouseButtonForNext(1), mouseButtonForPrev(4),
    newImageInOrigSize(false),
    actionCollection(0)
{
  cursorTimer = new QTimer(this);
  cursorTimer->setInterval(2000);
  cursorTimer->setSingleShot(true);
  connect(cursorTimer, SIGNAL(timeout()), this, SLOT(cursorHideSlot()));

  setFocusPolicy(Qt::StrongFocus);
  setBackground(Chequered);

  KAction *action;
  QList<QKeySequence> shortcuts;
  KShortcut shortcut;

  actionCollection = new KActionCollection(this);
  menu = new KMenu(this);

  action = actionCollection->addAction("rotateLeft", this, SLOT(rotateLeft()));
  action->setText(i18n("Rotate Left"));
  shortcuts.append(QKeySequence(Qt::CTRL + Qt::Key_L));
  shortcuts.append(QKeySequence(Qt::CTRL + Qt::Key_Left));
  action->setShortcuts(shortcuts);
  action->setIcon(KIcon("object-rotate-left"));
  menu->addAction(action);

  action = actionCollection->addAction("rotateRight", this, SLOT(rotateRight()));
  action->setText(i18n("Rotate Right"));
  shortcuts.clear();
  shortcuts.append(QKeySequence(Qt::CTRL + Qt::Key_R));
  shortcuts.append(QKeySequence(Qt::CTRL + Qt::Key_Right));
  action->setShortcuts(shortcuts);
  action->setIcon(KIcon("object-rotate-right"));
  menu->addAction(action);

  action = actionCollection->addAction("mirrorHorizontally", this, SLOT(mirrorHor()));
  action->setText(i18n("Mirror Horizontally"));
  action->setIcon(KIcon("object-flip-horizontal"));
  menu->addAction(action);

  action = actionCollection->addAction("mirrorVertically", this, SLOT(mirrorVer()));
  action->setText(i18n("Mirror Vertically"));
  action->setIcon(KIcon("object-flip-vertical"));
  menu->addAction(action);

  menu->addSeparator();
  action = actionCollection->addAction(KStandardAction::ActualSize, "origSize", this, SLOT(origSize()));
  action->setShortcut(QKeySequence(Qt::Key_O));
  menu->addAction(action);

  action = actionCollection->addAction(KStandardAction::FitToPage, "fitToPage", this, SLOT(fitToPage()));
  action->setShortcut(QKeySequence(Qt::Key_F));
  menu->addAction(action);

  action = actionCollection->addAction(KStandardAction::ZoomIn, "zoomIn", this, SLOT(zoomIn()));
  shortcut = action->shortcut();
  shortcut.setAlternate(QKeySequence(Qt::Key_Plus));
  action->setShortcut(shortcut);
  menu->addAction(action);

  action = actionCollection->addAction(KStandardAction::ZoomOut, "zoomOut", this, SLOT(zoomOut()));
  shortcut = action->shortcut();
  shortcut.setAlternate(QKeySequence(Qt::Key_Minus));
  action->setShortcut(shortcut);
  menu->addAction(action);

  fullScreenAction = KStandardAction::fullScreen(this, SLOT(toggleFullScreen()), this, this);
  shortcuts = fullScreenAction->shortcuts();
  shortcuts.append(QKeySequence(Qt::Key_Space));
  fullScreenAction->setShortcuts(shortcuts);
  menu->addAction(fullScreenAction);
  actionCollection->addAction("fullScreen", fullScreenAction);

  menu->addSeparator();

  action = actionCollection->addAction("properties", this, SLOT(properties()));
  action->setText(i18n("Properties..."));
  menu->addAction(action);

  action = actionCollection->addAction(KStandardAction::Save, "save", this, SLOT(save()));
  menu->addAction(action);

  action = actionCollection->addAction(KStandardAction::SaveAs, "saveAs", this, SLOT(saveAs()));
  menu->addAction(action);

  // copy to clipboard
  action = actionCollection->addAction(KStandardAction::Copy, "copy", this, SLOT(copy()));
  menu->addAction(action);

  action = actionCollection->addAction("deleteFile", this, SLOT(deleteFile()));
  action->setText(i18n("Delete File"));
  action->setIcon(KIcon("edit-delete"));
  action->setShortcut(QKeySequence(Qt::SHIFT + Qt::Key_Delete));
  menu->addAction(action);

  menu->addSeparator();

  action = actionCollection->addAction(KStandardAction::FirstPage, "first", this, SLOT(first()));
  action->setIcon(KIcon("go-first"));
  action->setText(i18n("First Image"));
  action->setToolTip(i18n("First Image"));
  menu->addAction(action);

  action = actionCollection->addAction(KStandardAction::Prior, "prior", this, SLOT(prior()));
  action->setIcon(KIcon("go-previous"));  // looks better than previous page icon
  action->setText(i18n("Previous Image"));
  action->setToolTip(i18n("Previous Image"));
  menu->addAction(action);

  action = actionCollection->addAction(KStandardAction::Next, "next", this, SLOT(next()));
  action->setIcon(KIcon("go-next"));
  action->setText(i18n("Next Image"));
  action->setToolTip(i18n("Next Image"));
  menu->addAction(action);

  action = actionCollection->addAction(KStandardAction::LastPage, "last", this, SLOT(last()));
  action->setIcon(KIcon("go-last"));
  action->setText(i18n("Last Image"));
  action->setToolTip(i18n("Last Image"));
  menu->addAction(action);

  menu->addSeparator();

  KMenu *settingsSubMenu = new KMenu(i18n("Settings"), this);
  menu->addMenu(settingsSubMenu);

  action = actionCollection->addAction("wheelToBrowse", this, SLOT(wheelSettingChanged(bool)));
  action->setText(i18n("Mouse Wheel for Browsing"));
  action->setCheckable(true);
  action->setChecked(wheelToBrowse = KGlobal::config()->group("").readEntry<bool>("wheelToBrowse", false));
  settingsSubMenu->addAction(action);

  // let the mousebutton for next/previous be configurable
  {
    mouseButtonForNext = KGlobal::config()->group("").readEntry<int>("mouseButtonNext", 1);

    KMenu *mouseButtonNext = new KMenu(i18n("Mouse Button for Next Image"), this);
    settingsSubMenu->addMenu(mouseButtonNext);

    QActionGroup *nextGroup = new QActionGroup(this);
    QSignalMapper *mapper = new QSignalMapper(this);
    connect(mapper, SIGNAL(mapped(int)), this, SLOT(mouseButtonNext(int)));

    for (int i = 1; i <= 7; i++)
    {
      action = new KAction(QString::number(i), this);
      action->setCheckable(true);
      nextGroup->addAction(action);
      mapper->setMapping(action, i);
      mouseButtonNext->addAction(action);
      connect(action, SIGNAL(triggered()), mapper, SLOT(map()));

      if ( i == mouseButtonForNext )
        action->setChecked(true);
    }
  }

  {
    mouseButtonForPrev = KGlobal::config()->group("").readEntry<int>("mouseButtonPrev", 4);

    KMenu *mouseButtonPrev = new KMenu(i18n("Mouse Button for Previous Image"), this);
    settingsSubMenu->addMenu(mouseButtonPrev);

    QActionGroup *nextGroup = new QActionGroup(this);
    QSignalMapper *mapper = new QSignalMapper(this);
    connect(mapper, SIGNAL(mapped(int)), this, SLOT(mouseButtonPrev(int)));

    for (int i = 1; i <= 7; i++)
    {
      action = new KAction(QString::number(i), this);
      action->setCheckable(true);
      nextGroup->addAction(action);
      mapper->setMapping(action, i);
      mouseButtonPrev->addAction(action);
      connect(action, SIGNAL(triggered()), mapper, SLOT(map()));

      if ( i == mouseButtonForPrev )
        action->setChecked(true);
    }
  }

  action = actionCollection->addAction(KStandardAction::KeyBindings, this, SLOT(configureShortcuts()));
  settingsSubMenu->addAction(action);

  action = actionCollection->addAction(KStandardAction::AboutApp, "about", this, SLOT(showAbout()));
  menu->addAction(action);

  menu->addSeparator();

  action = actionCollection->addAction(KStandardAction::Quit, "quit", qApp, SLOT(quit()));
  shortcuts = action->shortcuts();
  shortcuts.append(QKeySequence(Qt::Key_Return));
  action->setShortcuts(shortcuts);
  menu->addAction(action);

  // HACK: KShortcut can only handle 2 key sequences
  action = actionCollection->addAction(KStandardAction::Quit, "quit2", qApp, SLOT(quit()));
  shortcuts.clear();
  shortcuts.append(QKeySequence(Qt::Key_Enter));
  shortcuts.append(QKeySequence(Qt::Key_Escape));
  action->setShortcuts(shortcuts);

  actionCollection->associateWidget(this);
  actionCollection->readSettings();

  // AFTER the actions have been created
  overlay = new Overlay(this);
  overlay->hide();
  overlay->setSmall();
  setMouseTracking(true);
}

//--------------------------------------------------------------------------------

void ImageWidget::setFullScreen()
{
  KToggleFullScreenAction::setFullScreen(window(), true);
}

//--------------------------------------------------------------------------------

void ImageWidget::checkResize()
{
  if ( !fullScreenAction->isChecked() )
  {
    // mouse position percentage inside window
    double xp = (QCursor::pos().x() - frameGeometry().x()) / static_cast<double>(frameGeometry().width());
    double yp = (QCursor::pos().y() - frameGeometry().y()) / static_cast<double>(frameGeometry().height());

    // when the image is larger than the desktop, the window manager would reduce the window
    // size to fit onto the desktop which would lead to another smaller resize event
    // and the image would be scaled down; therefore limit newSize
    QSize newSize(qMin(image.width(), QApplication::desktop()->availableGeometry(this).width() - (frameSize().width() - width())),
                  qMin(image.height(), QApplication::desktop()->availableGeometry(this).height() - (frameSize().height() - height())));

    if ( newSize != size() )
    {
      // only undo "maximized" when we need a smaller window
      if ( ((image.width() < width()) || (image.height() < height())) &&
           (windowState() & Qt::WindowMaximized) )
        showNormal();  // if it was maximized, needed to be able to make window smaller again

      if ( !isVisible() )  // on startup, avoid to move the window so that kwin placement rules can apply
      {
        resize(newSize);
      }
      else if ( !(windowState() & Qt::WindowMaximized) )
      {
        QSize windowSize = newSize + (frameSize() - size());

        // move the window so that the mouse position is again at the point in the image before the resize
        QPoint newPos(QCursor::pos() - QPoint(windowSize.width() * xp, windowSize.height() * yp));

        QRect availableGeometry = QApplication::desktop()->availableGeometry(this);

        // move window to new position but keep it inside desktop
        if ( (newPos.x() + windowSize.width()) > availableGeometry.width() )
          newPos.setX(availableGeometry.width() - windowSize.width());

        if ( (newPos.y() + windowSize.height()) > availableGeometry.height() )
          newPos.setY(availableGeometry.height() - windowSize.height());

        if ( newPos.x() < availableGeometry.left() )
          newPos.setX(availableGeometry.left());

        if ( newPos.y() < availableGeometry.top() )
          newPos.setY(availableGeometry.top());

        // x,y is including window frame
        setGeometry(newPos.x() + geometry().x() - x(), newPos.y() + geometry().y() - y(),
                    newSize.width(), newSize.height());
      }
    }
  }

  // correct viewport
  adaptView();

  setTitle();  // adapt scale factor in window title/overlay
}

//--------------------------------------------------------------------------------

void ImageWidget::setTitle()
{
  QString title = QString("%1 [*] - (%2 x %3 : %4%)")
                          .arg(sourceUrl.fileName())
                          .arg(origImage.width())
                          .arg(origImage.height())
                          .arg(image.width() * 100 / origImage.width());

  int idx = fileNames.indexOf(sourceUrl.fileName());
  if ( idx != -1 )
  {
    title += i18n(" - %1 of %2")
                  .arg(idx+1)
                  .arg(fileNames.count());

    actionCollection->action("first")->setEnabled(idx > 0);
    actionCollection->action("prior")->setEnabled(idx > 0);
    actionCollection->action("next")->setEnabled(idx < (fileNames.count()-1));
    actionCollection->action("last")->setEnabled(idx < (fileNames.count()-1));
  }

  title += i18n(" in %1").arg(sourceUrl.directory());

  window()->setWindowTitle(title);

  // overlay text
  title = QString("%1 (%2 x %3 : %4%)\n")
                  .arg(sourceUrl.fileName())
                  .arg(origImage.width())
                  .arg(origImage.height())
                  .arg(image.width() * 100 / origImage.width());

  if ( idx != -1 )
  {
    title += i18n("%1 of %2 ")
                  .arg(idx+1)
                  .arg(fileNames.count());
  }

  title += i18n("in %1").arg(sourceUrl.directory());

  overlay->setText(title);
}

//--------------------------------------------------------------------------------

bool ImageWidget::showImage(const KUrl &url)
{
  if ( !url.isLocalFile() )
    return false;  // currently no remote urls allowed

  sourceUrl = url;

  if ( QFileInfo(url.path()).isDir() )  // not a file but a dir given
  {
    sourceUrl.addPath("dummy");
    getFileNames();
    if ( fileNames.isEmpty() )
      return false;

    return showImage(KUrl(sourceUrl.directory() + '/' + fileNames[0]));
  }

  if ( !image.load(sourceUrl.path()) )
    return false;

  setWindowModified(false);

  origImage = image;
  startPos = QPoint(0, 0);

  if ( newImageInOrigSize )
    origSize();
  else
    fitToPage();

  return true;
}

//--------------------------------------------------------------------------------

QSize ImageWidget::sizeHint() const
{
  return image.size();
}

//--------------------------------------------------------------------------------

void ImageWidget::paintEvent(QPaintEvent *)
{
  QPainter painter(this);
  painter.drawImage(startPos, image);
}

//--------------------------------------------------------------------------------

void ImageWidget::adaptView()
{
  if ( !fullScreenAction->isChecked() )
  {
    startPos.setX(qMin(0, qMax(startPos.x(), -(image.width() - width()))));
    startPos.setY(qMin(0, qMax(startPos.y(), -(image.height() - height()))));
  }
  else
  {
    // show image centered
    startPos.setX((QApplication::desktop()->screenGeometry(this).width() - image.width()) / 2);
    startPos.setY((QApplication::desktop()->screenGeometry(this).height() - image.height()) / 2);
  }
  update();
}

//--------------------------------------------------------------------------------

void ImageWidget::resizeEvent(QResizeEvent *event)
{
  overlay->hide();
  overlay->setFixedWidth(width());

  // if this is an event due to our calling of resize, we shall not scale the image again
  if ( (event->spontaneous() == 0) || fullScreenAction->isChecked() )
    return;

  QSize oldSize = image.size();
  image = origImage.scaled(event->size(), Qt::KeepAspectRatioByExpanding, transformationMode);

  if ( oldSize != image.size() )
    startPos = QPoint(0, 0);
  else
    adaptView();

  setTitle();
  update();
}

//--------------------------------------------------------------------------------

void ImageWidget::contextMenuEvent(QContextMenuEvent *)
{
  menu->exec(QCursor::pos());
}

//--------------------------------------------------------------------------------

void ImageWidget::moveStartPos(int dx, int dy)
{
  int newX = startPos.x() + dx;
  int newY = startPos.y() + dy;

  int maxX = 0;
  int maxY = 0;

  if ( fullScreenAction->isChecked() )
  {
    maxX = qMax(0, (QApplication::desktop()->screenGeometry(this).width() - image.width()) / 2);
    maxY = qMax(0, (QApplication::desktop()->screenGeometry(this).height() - image.height()) / 2);
  }

  startPos.setX(qMin(maxX, qMax(newX, -(image.width() - width()))));
  startPos.setY(qMin(maxY, qMax(newY, -(image.height() - height()))));

  update();
}

//--------------------------------------------------------------------------------

void ImageWidget::mouseMoveEvent(QMouseEvent *event)
{
  cursorTimer->start();
  setCursor(Qt::ArrowCursor);

  if ( event->buttons() == Qt::LeftButton )
  {
    moveStartPos(-(mousePos.x() - event->x()), -(mousePos.y() - event->y()));

    mousePos = event->pos();
  }
  else if ( event->buttons() == Qt::NoButton )
  {
    if ( (height() > (overlay->height() * 2)) && (width() >= 160) )
    {
      if ( event->y() < overlay->height() )
      {
        overlay->move(0, 0);
        overlay->show();
        cursorTimer->stop();
      }
      else if ( event->y() > (height() - overlay->height()) )
      {
        overlay->move(0, height() - overlay->height());
        overlay->show();
        cursorTimer->stop();
      }
      else
        overlay->hide();
    }
    else
      overlay->hide();
  }
}

//--------------------------------------------------------------------------------

void ImageWidget::leaveEvent(QEvent *)
{
  overlay->hide();
}

//--------------------------------------------------------------------------------

void ImageWidget::mousePressEvent(QMouseEvent *event)
{
  mousePressPos = mousePos = event->pos();
}

//--------------------------------------------------------------------------------

void ImageWidget::mouseReleaseEvent(QMouseEvent *event)
{
  if ( event->pos() == mousePressPos )  // click
  {
    if ( event->button() == mouseButtonForNext )
      next();
    else if ( event->button() == mouseButtonForPrev )
      prior();
  }
}

//--------------------------------------------------------------------------------

void ImageWidget::wheelEvent(QWheelEvent *event)
{
  event->accept();

  if ( (wheelToBrowse && (event->modifiers() == Qt::NoModifier)) ||
       (!wheelToBrowse && (event->modifiers() == Qt::ControlModifier)) )
  {
    KUrl oldUrl = sourceUrl;

    if ( event->delta() > 0 )
      prior();
    else
      next();

    return;
  }

  int numDegrees = event->delta() / 8;
  int numSteps = ::abs(numDegrees / 15);

  if ( numSteps == 0 )
    return;

  if ( numDegrees > 0 )
  {
    if ( (image.height() * 1.2 * numSteps) > (QApplication::desktop()->availableGeometry(this).width() * 2) )
      return;

    image = origImage.scaledToHeight(image.height() * 1.2 * numSteps, transformationMode);
  }
  else
  {
    if ( (image.height() / (1.2 * numSteps)) < 5 )
      return;

    image = origImage.scaledToHeight(image.height() / (1.2 * numSteps), transformationMode);
  }

  checkResize();
}

//--------------------------------------------------------------------------------

void ImageWidget::keyPressEvent(QKeyEvent *event)
{
  if ( event->modifiers() != Qt::NoModifier )
  {
    QWidget::keyPressEvent(event);
    return;
  }

  if ( event->key() == Qt::Key_Left )
    moveStartPos(20, 0);
  else if ( event->key() == Qt::Key_Right )
    moveStartPos(-20, 0);
  else if ( event->key() == Qt::Key_Up )
    moveStartPos(0, 20);
  else if ( event->key() == Qt::Key_Down )
    moveStartPos(0, -20);
  else if ( event->key() == Qt::Key_Home )
    moveStartPos(-startPos.x(), -startPos.y());
  else if ( event->key() == Qt::Key_End )
    moveStartPos(-image.width(), -image.height());
  else
  {
    QWidget::keyPressEvent(event);
    return;
  }
}

//--------------------------------------------------------------------------------

void ImageWidget::rotateLeft()
{
  origImage = origImage.transformed(QMatrix().rotate(-90));
  image = origImage.scaledToHeight(image.width(), transformationMode);
  checkResize();
  setWindowModified(true);
}

//--------------------------------------------------------------------------------

void ImageWidget::rotateRight()
{
  origImage = origImage.transformed(QMatrix().rotate(90));
  image = origImage.scaledToHeight(image.width(), transformationMode);
  checkResize();
  setWindowModified(true);
}

//--------------------------------------------------------------------------------

void ImageWidget::mirrorHor()
{
  origImage = origImage.mirrored(true, false);
  image = origImage.scaledToHeight(image.height(), transformationMode);
  checkResize();
  setWindowModified(true);
}

//--------------------------------------------------------------------------------

void ImageWidget::mirrorVer()
{
  origImage = origImage.mirrored(false, true);
  image = origImage.scaledToHeight(image.height(), transformationMode);
  checkResize();
  setWindowModified(true);
}

//--------------------------------------------------------------------------------

void ImageWidget::origSize()
{
  newImageInOrigSize = true;
  image = origImage;
  checkResize();
}

//--------------------------------------------------------------------------------

void ImageWidget::fitToPage()
{
  if ( fullScreenAction->isChecked() )
  {
    image = origImage.scaled(QApplication::desktop()->screenGeometry(this).width(),
                             QApplication::desktop()->screenGeometry(this).height(),
                             Qt::KeepAspectRatio, transformationMode);
  }
  else
  {
    // on startup and on toggling from fullscreen there is no window frame (decoration) so we will scale too large
    // Therefore limit the size by subtracting 40 pixels (which hopefully is at least as much as the window decoration will use)
    image = origImage.scaled(qMin(image.width(), QApplication::desktop()->availableGeometry(this).width() - 40 /*(frameSize().width() - width())*/),
                             qMin(image.height(), QApplication::desktop()->availableGeometry(this).height() - 40 /*(frameSize().height() - height())*/),
                             Qt::KeepAspectRatio, transformationMode);
  }

  newImageInOrigSize = false;
  checkResize();
}

//--------------------------------------------------------------------------------

void ImageWidget::zoomIn()
{
  // avoid memory overflow
  if ( (image.height() * 1.2) > (QApplication::desktop()->availableGeometry(this).width() * 2) )
    return;

  image = origImage.scaledToHeight(image.height() * 1.2, transformationMode);
  checkResize();
}

//--------------------------------------------------------------------------------

void ImageWidget::zoomOut()
{
  if ( (image.height() * 0.8) < 5 )
    return;

  image = origImage.scaledToHeight(image.height() * 0.8, transformationMode);

  checkResize();
}

//--------------------------------------------------------------------------------

void ImageWidget::properties()
{
  KPropertiesDialog::showDialog(sourceUrl, this);
}

//--------------------------------------------------------------------------------

void ImageWidget::save()
{
  if ( sourceUrl.isLocalFile() )
  {
    origImage.save(sourceUrl.pathOrUrl());
    setWindowModified(false);
  }
}

//--------------------------------------------------------------------------------

void ImageWidget::saveAs()
{
  QString target =
    KFileDialog::getSaveFileName(sourceUrl, KImageIO::mimeTypes(KImageIO::Writing).join(" "),
                                 this, i18n("Save Image As...")
#if KDE_IS_VERSION(4,4,0)
                                 , KFileDialog::ConfirmOverwrite
#endif
                                 );
  if ( !target.isEmpty() )
  {
    origImage.save(target);
  }
}

//--------------------------------------------------------------------------------

void ImageWidget::copy()
{
  QApplication::clipboard()->setImage(origImage);
}

//--------------------------------------------------------------------------------

void ImageWidget::deleteFile()
{
  if ( sourceUrl.isLocalFile() )
  {
    if ( KMessageBox::questionYesNo(this,
            i18n("Do you really want to delete file '%1'?").arg(sourceUrl.path()),
            i18n("Delete File"),
            KStandardGuiItem::yes(), KStandardGuiItem::no(),
            "dontAskAgainDelete") == KMessageBox::Yes )
    {
      getFileNames();  // so that the list holds this file to find the next one after deletion
      QFile theFile(sourceUrl.path());

      if ( !theFile.remove(sourceUrl.path()) )
      {
        KMessageBox::error(this, i18n("Could not delete the file.\nReason: %2").arg(theFile.errorString()));
      }
      else
      {
        // we have only the deleted file in the list - nothing else to do
        if ( fileNames.count() == 1 )
        {
          qApp->quit();
          return;
        }

        QString deletedFileName = sourceUrl.fileName();

        // show next file
        next();

        if ( deletedFileName == sourceUrl.fileName() )
        {
          // the deleted file was the last file - show previous
          prior();
        }

        // remove deleted file from list
        int idx = fileNames.indexOf(deletedFileName);
        if ( idx >= 0 )
        {
          fileNames.removeAt(idx);
          setTitle();  // update idx/count
        }
      }
    }
  }
}

//--------------------------------------------------------------------------------

void ImageWidget::toggleFullScreen()
{
  KToggleFullScreenAction::setFullScreen(this, fullScreenAction->isChecked());
  fitToPage();

  if ( fullScreenAction->isChecked() )
  {
    overlay->setLarge();
    setBackground(Black);
  }
  else
  {
    overlay->setSmall();
    setBackground(Chequered);
  }
}

//--------------------------------------------------------------------------------

void ImageWidget::setBackground(BgType t)
{
  QPalette pal = palette();

  if ( t == Black )
    pal.setBrush(backgroundRole(), Qt::black);
  else
  {
    /* XPM */
    static const char *chequered[]={
    "16 16 2 1",
    ". c #c0c0c0",
    "# c #ffffff",
    "........########",
    "........########",
    "........########",
    "........########",
    "........########",
    "........########",
    "........########",
    "........########",
    "########........",
    "########........",
    "########........",
    "########........",
    "########........",
    "########........",
    "########........",
    "########........"};

    pal.setBrush(backgroundRole(), QImage(chequered));
  }

  setPalette(pal);
}

//--------------------------------------------------------------------------------

void ImageWidget::showAbout()
{
  KAboutApplicationDialog dialog(KCmdLineArgs::aboutData());
  dialog.exec();
}

//--------------------------------------------------------------------------------

bool naturalCompareLessThan(const QString &s1, const QString &s2)
{
  return KStringHandler::naturalCompare(s1, s2, Qt::CaseInsensitive) < 0;
}

//--------------------------------------------------------------------------------

void ImageWidget::getFileNames()
{
  if ( fileNames.isEmpty() )  // only load list once to be fast
  {
    fileNames = QDir(sourceUrl.directory()).
                  entryList(KImageIO::pattern().split(' ', QString::SkipEmptyParts),
                            QDir::Readable | QDir::Files);

    // sort by name using natural sorting (e.g. file1, file2, file10, file20)
    qSort(fileNames.begin(), fileNames.end(), naturalCompareLessThan);
  }
}

//--------------------------------------------------------------------------------

void ImageWidget::prior()
{
  getFileNames();
  int idx = fileNames.indexOf(sourceUrl.fileName());
  while ( (idx > 0) && !showImage(KUrl(sourceUrl.directory() + '/' + fileNames[--idx])) )
    prior();
}

//--------------------------------------------------------------------------------

void ImageWidget::next()
{
  getFileNames();
  int idx = fileNames.indexOf(sourceUrl.fileName());
  if ( idx >= 0 )
  {
    while ( (idx < (fileNames.count() - 1)) &&
            !showImage(KUrl(sourceUrl.directory() + '/' + fileNames[++idx])) )
      next();
  }
}

//--------------------------------------------------------------------------------

void ImageWidget::first()
{
  getFileNames();
  showImage(KUrl(sourceUrl.directory() + '/' + fileNames.first()));
}

//--------------------------------------------------------------------------------

void ImageWidget::last()
{
  getFileNames();
  showImage(KUrl(sourceUrl.directory() + '/' + fileNames.last()));
}

//--------------------------------------------------------------------------------

void ImageWidget::wheelSettingChanged(bool checked)
{
  wheelToBrowse = checked;

  KGlobal::config()->group("").writeEntry("wheelToBrowse", checked);
  KGlobal::config()->group("").sync();
}

//--------------------------------------------------------------------------------

void ImageWidget::mouseButtonNext(int num)
{
  mouseButtonForNext = num;

  KGlobal::config()->group("").writeEntry("mouseButtonNext", num);
  KGlobal::config()->group("").sync();
}

//--------------------------------------------------------------------------------

void ImageWidget::mouseButtonPrev(int num)
{
  mouseButtonForPrev = num;

  KGlobal::config()->group("").writeEntry("mouseButtonPrev", num);
  KGlobal::config()->group("").sync();
}

//--------------------------------------------------------------------------------

void ImageWidget::configureShortcuts()
{
  KShortcutsDialog::configure(actionCollection, KShortcutsEditor::LetterShortcutsAllowed, this);
}

//--------------------------------------------------------------------------------

QAction *ImageWidget::getAction(const QString &name) const
{
  return actionCollection->action(name);
}

//--------------------------------------------------------------------------------

void ImageWidget::cursorHideSlot()
{
  setCursor(Qt::BlankCursor);
}

//--------------------------------------------------------------------------------
